from ctypes import *
from ctypes.wintypes import *
import os
#import the rewritten dll and set its path
pt_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "protalker"))
regtts = cdll[os.path.join(pt_path, "regtts.dll")]
regtts.setpath(pt_path)

class Params(Structure):
 _fields_ = [
 ('rate', c_int), #from 4 to 13 inclusive
 ('pitch', c_int), #50 to 170 inclusive
 ('i3', c_int), #0 to 9 but not sure what it does.
 ('accent', c_int), #0 to 9 inclusive
 ('font', c_int), #voice font. 1 or 2
 ('u2', c_int),
 ('off20', WORD),
 ]

class PTError (Exception): pass

cb = None
PTCallback = WINFUNCTYPE(c_int, c_int, c_int)
@PTCallback
def ptcallback(addr, bytes):
 data = string_at(addr, bytes)
 return cb(data)

class Synth(object):
 def __init__(self):
  self.dll = windll[os.path.join(pt_path, "tts_mgr.dll")]
  self.handle = c_int()
  self.params = Params()
  self.params.rate = 13
  self.params.font = 1
  self.params.pitch = 98
  self.params.u2 = 1
  self.params.accent = 5
  self.params.off20 = 77
  self.dll.spcSynInit(61442, 0, pointer(self.handle), "user")
  self.dll.spcSynCntl(self.handle.value, pointer(self.params))
  self._boost = 100
  self.dll.setboost.argtypes = (c_float,)

 def say(self, text, cb2):
  global cb
  text = text.strip()
  if text == '': return
  cb = cb2
  self.dll.spcSynCntl(self.handle.value, pointer(self.params))
  ret = self.dll.spcSynSpeak(self.handle.value, text.encode('cp932', 'replace'), 0, 0, 999, ptcallback)
  #if ret != 0:
   #raise PTError, ret

 def free(self):
  self.dll.spcSynTerm(self.handle.value)

 @property
 def rate(self):
  return self.params.rate-4

 @rate.setter
 def rate(self, val):
  self.params.rate = val+4

 @property
 def pitch(self):
  return self.params.pitch

 @pitch.setter
 def pitch(self, val):
  self.params.pitch = val

 @property
 def accent(self):
  return self.params.accent

 @accent.setter
 def accent(self, val):
  self.params.accent = val

 @property
 def boost(self):
  return self._boost

 @boost.setter
 def boost(self, val):
  self._boost = val
  self.dll.setboost(val/100.0)
