﻿# -*- encoding: utf-8 -*-
import os
import sys
from synthDriverHandler import SynthDriver,VoiceInfo,BooleanSynthSetting, NumericSynthSetting
from collections import OrderedDict

filepath = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(filepath, "../ptenglish"))
import _protalker
del sys.path[-1]
from cStringIO import StringIO
data = StringIO()

isSpeaking = False
lastindex = None
from ctypes import *
from synthDriverHandler import SynthDriver
import config
import nvwave
import threading
import logging
import Queue
bgq = Queue.Queue()
class BgThread(threading.Thread):
 def __init__(self, q):
  threading.Thread.__init__(self)
  self.setDaemon(True)
  self.q = q

 def run(self):
  try:
   while True:
    func, args, kwargs = self.q.get()
    if not func:
     break
    func(*args, **kwargs)
    self.q.task_done()
  except:
   logging.error("bgThread.run", exc_info=True)

def _bgExec(q, func, *args, **kwargs):
 q.put((func, args, kwargs))

player = None
def ptcallback(bytes):
	if not isSpeaking: return 1
	data.write(bytes)
	if isSpeaking and data.tell() >= 3000*2: #*2 for 16-bit mono audio
		player.feed(data.getvalue())
		data.truncate(0)
	return 1

def speak(synth, text):
	global isSpeaking
	isSpeaking = True
	data.truncate(0)
	synth.say(text, ptcallback)
	if isSpeaking:
		if data.tell() > 0:
			player.feed(data.getvalue())
		isSpeaking = False
		player.idle()

def setindex(val):
	global lastindex
	lastindex = val

class SynthDriver(SynthDriver):
	supportedSettings = (
	SynthDriver.RateSetting(minStep=10),
	SynthDriver.PitchSetting(),
	SynthDriver.VoiceSetting(),
	NumericSynthSetting("ptRateBoost", _("ProTALKER rate boost"), normalStep=10),
	SynthDriver.InflectionSetting(minStep=10),
)
	name = 'ptjp'
	description = "ProTALKER"

	@classmethod
	def check(cls):
		return True

	def __init__(self, *args, **kwargs):
		super(SynthDriver, self).__init__(*args, **kwargs)
		global player
		self.synth = _protalker.Synth()
		player = nvwave.WavePlayer(channels=1, samplesPerSec=22050, bitsPerSample=16, outputDevice=config.conf["speech"]["outputDevice"])
		self.bgt = BgThread(bgq)
		self.bgt.start()
		self.rate = 100
		self.ptRateBoost = 0
		#Voices were added later, so people with old configs don't have it
		if 'ptjp' in config.conf['speech'] and 'voice' not in config.conf['speech']['ptjp']:
			config.conf['speech']['ptjp']['voice'] = u'1'



	def speakText(self, text, index=None):
		_bgExec(bgq, speak, self.synth, text)
		if index is not None:
			_bgExec(bgq, setindex, index)


	def cancel(self):
		global isSpeaking
		isSpeaking = False
		clear_queue(bgq)
		player.stop()
		data.truncate(0)

	def terminate(self):
		global player, isSpeaking
		isSpeaking = False
		_bgExec(bgq, None)
		self.bgt.join()
		player.close()
		player = None
		self.synth.free()

	def _get_rate(self):
		return self._paramToPercent(self.synth.rate, 0, 9)

	def _set_rate(self, value):
		self.synth.rate = self._percentToParam(value, 0, 9)

	def _get_inflection(self):
		return self._paramToPercent(self.synth.accent, 0, 9)

	def _set_inflection(self, value):
		self.synth.accent = self._percentToParam(value, 0, 9)

	def _get_pitch(self):
		return self._paramToPercent(self.synth.pitch, 50, 170)

	def _set_pitch(self, value):
		new = self._percentToParam(value, 50, 170)
		_bgExec(bgq, setattr, self.synth, "pitch", new)

	def _get_ptRateBoost(self):
		return self._paramToPercent(self.synth.boost, 100, 200)

	def _set_ptRateBoost(self, value):
		self.synth.boost = self._percentToParam(value, 100, 200)

	def _get_lastIndex(self):
		return lastindex

	def pause(self, switch):
		player.pause(switch)

	def _get_availableVoices(self):
		availableVoices = OrderedDict()
		availableVoices[u"1"] = VoiceInfo(u"1", u"読み秀君", u"ja")
		availableVoices[u"2"] = VoiceInfo(u"2", u"読み子ちゃん", u"ja")
		return availableVoices

	def _get_voice(self):
		return str(self.synth.params.font)

	def _set_voice(self, value):
		self.synth.params.font = int(value)

	#For NVDAJP's extended NVDAHelper
	def isSpeaking(self):
		global isSpeaking
		return isSpeaking

def clear_queue(q):
 lst = []
 try:
  while True:
   item = q.get_nowait()
   if item[0] != speak:
    lst.append(item)
 except:
  pass
 for item in lst:
   q.put(item)
   